<?php

namespace App\Http\Controllers\Api\Salon;

use App\Http\Controllers\Controller;
use App\Http\Resources\BankResource;
use App\Http\Resources\SubscriptionResource;
use App\Models\Bank;
use App\Models\Setting;
use App\Models\Subscription;
use App\Models\Transfer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use JWTAuth;

class SubscriptionsController extends Controller
{

    public function subscriptions()
    {
        $subscriptions = Subscription::all();
        return SubscriptionResource::collection($subscriptions)->additional(['value' => true]);
    }

    public function check_subscription()
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong_token'), 'code' => 401]);
        }

        if (!$user->subscription_expiry) {
            return response()->json(['value' => false, 'msg' => 'برجاء الاشتراك']);
        } elseif (Carbon::now() > Carbon::parse($user->subscription_expiry)) {
            return response()->json(['value' => false, 'msg' => 'تم انتهاء الاشتراك برجاء التجديد']);
        }

        $now = Carbon::now();
        $date = Carbon::parse($user->subscription_expiry);
        $diff = $date->diffInDays($now);

        return response()->json(['value' => true, 'data' => "باقى لديك على انتهاء الاشتراك $diff ايام"]);

    }

    public function subscribe(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'type' => 'required|in:bank,visa',
            'bank_id' => 'required_if:type,bank|exists:banks,id',
            'subscription_id' => 'required|exists:subscriptions,id',
            'image' => 'required_if:type,bank|image',
            'amount' => 'required|numeric',
        ]);
        if ($validator->passes()) {
            $transfer = new Transfer();
            $transfer->bank_id = $request->bank_id;
            $transfer->type = $request->type;
            $transfer->status = $request->type == 'bank' ? 'waiting' : 'accept';
            $transfer->user_id = $user->id;
            $transfer->amount = $request->amount;
            $transfer->subscription_id = $request->subscription_id;
            if ($request->hasFile('image')) $transfer->setImageAttribute();
            $transfer->save();
            if ($request->type == 'bank') {
                return response()->json(['value' => true, 'data' => 'تم ارسال التحويل بنجاح وبانتظار المراجعه', 'code' => 200]);
            }
            $user->subscription_expiry = Carbon::now()->addDays($transfer->subscription->duration)->toDateString();
            $user->subscription_id = $request->subscription_id;
            $user->update();
            return response()->json(['value' => true, 'data' => 'تم الاشتراك بنجاح', 'code' => 200]);

        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function banks()
    {
        $banks = Bank::all();
        return BankResource::collection($banks)->additional(['value' => true]);
    }

}
