<?php

namespace App\Http\Controllers\Api\Salon;

use App\Http\Controllers\Controller;
use App\Http\Resources\PackageResource;
use App\Http\Resources\SalonCategories;
use App\Http\Resources\SalonResource;
use App\Http\Resources\SalonTimeResource;
use App\Http\Resources\ServicesResource;
use App\Http\Resources\UserSalonResource;
use App\Models\Package;
use App\Models\Salon;
use App\Models\SalonCategory;
use App\Models\SalonImage;
use App\Models\SalonTime;
use App\Models\Service;
use App\Models\UserTokens;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;


class UsersController extends Controller
{

    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required',
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',

        ]);
        if ($validator->passes()) {
            $auth = ['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'active', 'type' => 'client'];
            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')], 422);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }
            $user = User::whereMobile($request['mobile'])->first();

            $check = UserTokens::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->update();
            } else {
                UserTokens::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register_step1(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [


            'mobile' => 'required|unique:users,mobile|digits:9',
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'username' => 'required|unique:users,username',
            'email' => 'required|email|unique:users,email',
            'city_id' => 'required|exists:cities,id',
            'field_id' => 'required|exists:fields,id',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['username'] = $request['username'];
            $user['city_id'] = $request['city_id'];
            $user['field_id'] = $request['field_id'];
            $user['lat'] = $request['lat'];
            $user['lng'] = $request['lng'];
            $user['password'] = bcrypt($request['password']);
            $user['type'] = 'salon';
//            $user['code'] = rand(1111,9999);
            $user['code'] = 1111;
            $user['status'] = 'not_active';
            $user['admin_approve'] = 'false';
            $user->save();
            $msg = "عميلنا العزيز كود التفعيل الخاص بكم هو $user->code";
            shamel_sms($user->mobile, $msg);

            $user_salon = new Salon();
            $user_salon->user_id = $user->id;
            $user_salon->city_id = $request['city_id'];
            $user_salon->save();

            $check = UserTokens::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                $check->user_id = $user->id;
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                UserTokens::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {

            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register_step2(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'type' => ['required', 'in:freelance,salon'],
            'address' => 'required',
            'details' => 'required',
            'always_open' => ['required', 'in:true,false']
        ]);
        if ($validator->passes()) {
            $salon = Salon::where('user_id', $user->id)->first();
            $salon['type'] = $request['type'];
            $salon['address'] = $request['address'];
            $salon['details'] = $request['details'];
            $salon['always_open'] = $request['always_open'];
            $salon->update();

            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register_step3(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'day' => 'required',
            'from' => 'required',
            'to' => 'required',
            'open_status' => ['required', 'in:open,close']
        ]);
        if ($validator->passes()) {
            $salon_time = new SalonTime();
            $salon_time['salon_id'] = $user->salon->id;
            $salon_time['day'] = $request['day'];
            $salon_time['from'] = $request['from'];
            $salon_time['to'] = $request['to'];
            $salon_time['open_status'] = $request['open_status'];
            $salon_time->save();

            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function register_step4(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'salon_title' => 'required',
            'image' => 'required',
            'salon_images' => 'required'
        ]);
        if ($validator->passes()) {
            $salon = Salon::where('user_id', $user->id)->first();
            $salon->salon_title = $request->salon_title;
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }

            $user->update();
            $salon->update();
            if ($request->salon_images) {
                foreach ($request->salon_images as $salon_image) {
                    $images = new SalonImage();
                    $images->salon_id = $user->salon->id;
                    $images->setImageAttribute($salon_image);
                    $images->save();
                }
            }
            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register_step5(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'categories' => 'required'
        ]);
        if ($validator->passes()) {
            SalonCategory::where('salon_id')->delete();
            $categories = explode(',', $request->categories);
            foreach ($categories as $category) {
                $salon_category = new SalonCategory();
                $salon_category->salon_id = $user->salon->id;
                $salon_category->category_id = $category;
                $salon_category->save();
            }
            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function update_salon_personal_data(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'username' => 'unique:users,username,' . $user->id,
            'email' => 'email|unique:users,email,' . $user->id,
            'mobile' => 'digits:9|unique:users,mobile,' . $user->id,

            'city_id' => 'nullable|exists:cities,id',
        ]);
        if ($validator->passes()) {
            $user->username = $request->username ? $request['user_name'] : $user->username;
            $user->mobile = $request->mobile ? $request['username'] : $user->mobile;
            $user->email = $request->email ? $request['email'] : $user->email;
            $user->city_id = $request->city_id ? $request['city_id'] : $user->city_id;
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }
            $user->update();
//            $salon = Salon::find($user->salon->id);
            Salon::whereUserId($user->id)->update([
                'address' => $request['address']
            ]);
//            $salon->address = $request->address;
//            $salon->update;
            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }


    public function salon_all_services($id)
    {
        $data = ServicesResource::collection(Service::whereSalonId($id)->get());
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function update_salon_data(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'salon_title' => 'required',
            'type' => 'required|in:freelance,salon',
//            'image' => 'image',
            'always_open' => 'required|in:true,false'
        ]);
        if ($validator->passes()) {
            $salon = Salon::find($user->salon->id);
            $salon['salon_title'] = $request['salon_title'];
            $salon['type'] = $request['type'];
            if ($request->hasFile('image')) {
                $salon->setImageAttribute();
            }
            $salon->update();
            if ($request->salon_images) {
                SalonImage::where('salon_id', $user->salon->id)->delete();
                foreach ($request->salon_images as $salon_image) {
                    $images = new SalonImage();
                    $images->salon_id = $user->salon->id;
                    $images->setImageAttribute($salon_image);
                    $images->save();
                }
            }
            $data = new UserSalonResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function update_notification(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'status' => ['required', 'in:true,false']
        ]);


        if ($validator->passes()) {
            Salon::where('id', $user->salon->id)->update([
                'receive_notifications' => $request['status']
            ]);
        }

        return response()->json(['value' => true, 'data' => 'تم التعديل بنجاح']);

    }

    public function delete_time($id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        SalonTime::destroy($id);
        $rows = SalonTime::where('salon_id', $user->salon->id)->paginate(10);
        return SalonTimeResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function update_time(Request $request, $id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'day' => 'required',
            'from' => 'required',
            'to' => 'required',
            'open_status' => ['required', 'in:open,close']
        ]);
        if ($validator->passes()) {

            $salon_time = SalonTime::find($id);
            $salon_time['day'] = $request['day'];
            $salon_time['from'] = $request['from'];
            $salon_time['to'] = $request['to'];
            $salon_time['open_status'] = $request['open_status'];
            $salon_time->update();
            $rows = SalonTime::where('salon_id', $user->salon->id)->paginate(10);
            return SalonTimeResource::collection($rows)->additional(['value' => true, 'code' => 200]);

        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function salon_times()
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $rows = SalonTime::where('salon_id', $user->salon->id)->paginate(10);
        return SalonTimeResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function add_package(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'duration_type' => 'required|in:days,months',
            'price' => 'required|numeric',
            'discount_percent' => 'nullable|numeric',
            'description' => 'required',
            'duration' => 'required|numeric',
            'place' => 'required|in:home,salon'
        ]);
        if ($validator->passes()) {
            $package = new Package();
            $package->salon_id = $user->salon->id;
            $package->title = $request->title;
            $package->duration_type = $request->duration_type;
            $package->price = $request->price;
            $package->place = $request->place;
            $package->discount_percent = $request->discount_percent;
            $package->description = $request->description;
            $package->duration = $request->duration;
            if ($request->duration_type == 'days') {
                $package->finish_date = Carbon::now()->addDays($request->duration)->toDateString();
            } elseif ($request->duration_type == 'months') {
                $package->finish_date = Carbon::now()->addMonths($request->duration)->toDateString();
            }
            $package->save();

            return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function update_package(Request $request, $id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'duration_type' => 'required|in:days,months',
            'price' => 'required|numeric',
            'discount_percent' => 'nullable|numeric',
            'description' => 'required',
            'duration' => 'required|numeric',
            'place' => 'in:home,salon'
        ]);
        if ($validator->passes()) {
            $package = Package::find($id);
            $package->salon_id = $user->salon->id;
            $package->title = $request->title;
            $package->duration_type = $request->duration_type;
            $package->price = $request->price;
            $package->place = $request->place ? $request['place'] : $package->place;
            $package->discount_percent = $request->discount_percent;
            $package->description = $request->description;
            $package->duration = $request->duration;
            if ($request->duration_type == 'days') {
                $package->finish_date = Carbon::now()->addDays($request->duration)->toDateString();
            } elseif ($request->duration_type == 'months') {
                $package->finish_date = Carbon::now()->addMonths($request->duration)->toDateString();
            }
            $package->update();

            return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function delete_package(Request $request, $id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $package = Package::find($id);
        if ($user->salon->id == $package->salon_id) {
            $package->delete();
        } else {
            return response()->json(['value' => false, 'msg' => 'المستخدم ليس صاحب هذا العرض', 'code' => 401]);
        }
        return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
    }

    public function add_service(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'duration' => 'required|numeric',
            'price' => 'required|numeric',
            'discount_percent' => 'nullable|numeric',
            'image' => 'image',
            'category_id' => 'required|exists:categories,id',
            'place' => 'required|in:salon,home'
        ]);
        if ($validator->passes()) {
            $service = new Service();
            $service->salon_id = $user->salon->id;
            $service->title = $request->title;
            $service->price = $request->price;
            $service->discount_percent = $request->discount_percent;
            $service->duration = $request->duration;
            $service->place = $request->place;
            $service->category_id = $request->category_id;
            if ($request->hasFile('image')) {
                $service->setImageAttribute();
            }
            $service->save();

            return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function salon_categories($id)
    {
        $salon = Salon::find($id);

        $data = new SalonCategories($salon);

        return response()->json(['value' => true, 'data' => $data]);

    }

    public function packages($id)
    {
        $salon = Salon::find($id);
        if ($salon) {
            $data = PackageResource::collection($salon->packages);
        } else {
            $data = [];
        }


        return response()->json(['value' => true, 'data' => $data]);

    }

    public function update_service(Request $request, $id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'duration' => 'required|numeric',
            'price' => 'required|numeric',
            'discount_percent' => 'nullable|numeric',
            'image' => 'image|nullable',
            'category_id' => 'required|exists:categories,id',
            'place' => 'required|in:salon,home'
        ]);
        if ($validator->passes()) {
            $service = Service::find($id);
            $service->salon_id = $user->salon->id;
            $service->title = $request->title;
            $service->price = $request->price;
            $service->discount_percent = $request->discount_percent;
            $service->duration = $request->duration;
            $service->place = $request->place;
            $service->category_id = $request->category_id;
            if ($request->hasFile('image')) {
                $service->setImageAttribute();
            }
            $service->update();

            return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }

    public function delete_service(Request $request, $id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $service = Service::find($id);
        if ($user->salon->id == $service->salon_id) {
            $service->delete();
        } else {
            return response()->json(['value' => false, 'msg' => 'المستخدم ليس صاحب هذا العرض', 'code' => 401]);
        }
        return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
    }

    public function update_password(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'new_password' => 'required|confirmed',
            'new_password_confirmation' => 'required'
        ]);
        if ($validator->passes()) {
            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['new_password']);
                $user->update();
                return response()->json(['value' => true, 'data' => new UserSalonResource($user), 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match'), 'code' => 422]);
            }

        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }


    public function search_salon(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {

            $lat = $request['lat'];
            $lng = $request['lng'];

            $rows = User::select(\DB::raw('*, ( 6367 * acos( cos( radians(' . $lat . ') ) * cos( radians( lat ) ) * cos( radians( lng ) - radians(' . $lng . ') ) + sin( radians(' . $lat . ') ) * sin( radians( lat ) ) ) ) AS distance'))
                ->having('distance', '<', 100)->orderBy('distance')->where('type', 'salon')->where('admin_approve', 'true')->where('status', 'active')->paginate(8);
            return SalonResource::collection($rows)->additional(['value' => true, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

}
