<?php

namespace App\Models;

use App\User;
use Illuminate\Database\Eloquent\Model;

class Salon extends Model
{

    protected $table = 'salons';
    public $timestamps = true;
    protected $fillable = array('user_id', 'lat', 'lng', 'type', 'details', 'address', 'always_open', 'receive_notifications', 'salon_title', 'image', 'wallet','city_id','rate','order_counts');





    public function deleteImage()
    {
        @unlink(public_path('images/salons/' . $this->attributes['image']));

    }

    public function setImageAttribute()
    {
        $rand = rand(111, 999);
        $file = request()->file('image');
        $destinationPath = 'images/users/';
        $filename = $rand . '_' . time() . '.' . $file->getClientOriginalExtension();
        $file->move($destinationPath, $filename);
        $this->attributes['image'] = $filename;
    }



    public function user()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function images()
    {
        return $this->hasMany(SalonImage::class, 'salon_id');
    }


    protected $appends = ['total_rate'];

    public function getTotalRateAttribute()
    {
        $sum = SalonComment::where('salon_id', $this->id)->sum('rate');
        $count = SalonComment::where('salon_id', $this->id)->count();
        if ($sum == 0) {
            return 0;
        } else {
            return (integer)($sum / $count);
        }
    }

    public function custom_categories($flag)
    {
        $data = [];
        $categories = $this->categories()->get();
        foreach ($categories as $category) {
            $arr['id'] = $category->id;
            $arr['name'] = $category->name;
            $arr['services'] = [];
            $services = Service::where('salon_id', $this->id)->where('category_id', $category->id)->get();
            foreach ($services->where('place',$flag) as $service) {
                $ser['id'] = $service->id;
                $ser['title'] = $service->title ? $service->title : '';
                $ser['duration'] = $service->duration ? $service->duration : '';
                $ser['place'] = $service->place ? $service->place : '';
                $ser['duration_type'] = $service->duration_type ? $service->duration_type : '';
                if ($service->discount_percent) {
                    $price = (integer)($service->price - ($service->price * $service->discount_percent / 100));
                } else {
                    $price = $service->price;
                }
                $ser['price'] = (integer)$price;
                $ser['discount_percent'] = $service->discount_percent ? $service->discount_percent : '';
                $ser['image'] = $service->image ? $service->image : '';
                $arr['services'][] = $ser;
            }
            $data[] = $arr;
        }
        return $data;
    }



    public function categories()
    {
        return $this->belongsToMany(Category::class, 'salon_categories', 'salon_id', 'category_id');
    }

    public function services()
    {
        return $this->hasMany(Service::class, 'salon_id');
    }

    public function packages()
    {
        return $this->hasMany(Package::class, 'salon_id');
    }

    public function comments()
    {
        return $this->hasMany(SalonComment::class, 'salon_id');
    }

    public function salon_times()
    {
        return $this->hasMany(SalonTime::class, 'salon_id');
    }

    public function favourite_users()
    {
        return $this->belongsToMany(User::class, 'favourites', 'salon_id', 'user_id');
    }

    public function rooms()
    {
        return $this->hasMany(Room::class, 'salon_id');
    }

    public function accounts()
    {
        return $this->hasMany(SalonAccount::class, 'salon_id');
    }

    public function bank_transfers()
    {
        return $this->hasMany(SalonBankTransfer::class, 'salon_id');
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'salon_id');
    }

}
