<?php

namespace App;

use App\Models\Chats;
use App\Models\City;
use App\Models\Notification;
use App\Models\Order;
use App\Models\Room;
use App\Models\Salon;
use App\Models\SalonComment;
use App\Models\Service;
use App\Models\Subscription;
use App\Models\UserTokens;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Tymon\JWTAuth\Contracts\JWTSubject;

class User extends Authenticatable implements JWTSubject
{
    use Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array('username', 'mobile', 'email', 'city_id', 'password', 'type', 'status', 'admin_approve', 'code', 'image', 'lat', 'lng','field_id');

    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }
    public function setImageAttribute()
    {
        $rand = rand(111,999);
        $file = request()->file('image');
        $destinationPath = 'images/users/';
        $filename = $rand .'_'.time().'.'.$file->getClientOriginalExtension();
        $file->move($destinationPath, $filename);
        $this->attributes['image'] = $filename;
    }


    public function custom_categories($flag)
    {
        $data = [];
        $categories = $this->salon->categories()->whereHas('services',function ($q) use ($flag){
            $q->where('place',$flag);
        })->get();
        foreach ($categories as $category) {
            $arr['id'] = $category->id;
            $arr['name'] = $category->name;
            $arr['services'] = [];
            $services = Service::where('salon_id', $this->salon->id)->where('category_id', $category->id)->get();
            foreach ($services->where('place',$flag) as $service) {
                $ser['id'] = $service->id;
                $ser['title'] = $service->title ? $service->title : '';
                $ser['duration'] = $service->duration ? $service->duration : '';
                $ser['place'] = $service->place ? $service->place : '';
                $ser['duration_type'] = $service->duration_type ? $service->duration_type : '';
                if ($service->discount_percent) {
                    $price = $service->price - ($service->price * $service->discount_percent / 100);
                } else {
                    $price = $service->price;
                }
                $ser['price'] = (integer)$price;
                $ser['discount_percent'] = $service->discount_percent ? $service->discount_percent : '';
                $ser['image'] = $service->image ? $service->image : '';
                $arr['services'][] = $ser;
            }
            $data[] = $arr;
        }
        return $data;
    }


    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    //relations

    public function subscription()
    {
        return $this->belongsTo(Subscription::class, 'subscription_id');
    }

    public function city()
    {
        return $this->belongsTo(City::class, 'city_id');
    }

    public function tokens()
    {
        return $this->hasMany(UserTokens::class, 'user_id');
    }

    public function salon()
    {
        return $this->hasOne(Salon::class, 'user_id');
    }

    public function comments()
    {
        return $this->hasMany(SalonComment::class, 'user_id');
    }

    public function favourite_salons()
    {
        return $this->belongsToMany(Salon::class, 'favourites', 'user_id', 'salon_id');
    }

    public function rooms()
    {
        return $this->hasMany(Room::class, 'user_id');
    }

    public function sending_messages()
    {
        return $this->hasMany(Chats::class, 'sender_id');
    }

    public function receive_messages()
    {
        return $this->hasMany(Chats::class, 'receiver_id');
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'user_id');
    }

    public function notifications()
    {
        return $this->hasMany(Notification::class, 'user_id');
    }

}
